#include "kinesis.h"
#include "action_layer.h"
#include "eeconfig.h"

#define _CMD   0 // Base Colemak Mod-DH
#define _QW    1 // Base QWERTY
#define _CG    2 // Colemak Mod-DH gaming layer
#define _QG    3 // QWERTY gaming layer
#define _NM    4 // Number layer
#define _MD    5 // Media Layer
#define _KP    6 // KP layer
#define _LY    7 // Layer switcher
#define _FN    8 // Function layer
#define _FN2   9 // Function layer (identical as _FN; used to deal with minor key interaction issue)

#define _______ KC_TRNS
#define XXXXXXX KC_NO

//Tap Dance Declarations
enum {
  LPN_LBC,
  RPN_RBC
};

qk_tap_dance_action_t tap_dance_actions[] = {
[LPN_LBC] = ACTION_TAP_DANCE_DOUBLE(KC_LPRN, KC_LBRC),
[RPN_RBC] = ACTION_TAP_DANCE_DOUBLE(KC_RPRN, KC_RBRC)
};

/*

        Function Keys on All Layers (Keypad toggles):
	,-----------------------------------------------------------------.
	|  ESC |  F1  |  F2  |  F3  |  F4  |  F5  |  F6  |   F7   |  F8   |
 	`-----------------------------------------------------------------'
	,---------------------------------------------------------------- --------------.
	|  F9  |  F10 |  F11 |  F12 | PScr | SLck | Paus | Keypad | Layer/              |
	|      |      |      |      |      |      |      |        | RESET (in Fn layer) |
	`-------------------------------------------------------------------------------'

        Colemak Mod-DH layer:
	,-------------------------------------------.,-------------------------------------------.
	|   =    |   1  |   2  |   3  |   4  |   5  ||   6  |   7  |   8  |   9  |   0  |   -    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| Tab    |   Q  |   W  |   F  |   P  |   B  ||   J  |   L  |   U  |   Y  |   ;  |   \    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| Ctl/Esc|   A  |   R  |   S  |   T  |   G  ||   M  |   N  |   E  |   I  |   O  |   '    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| (    [ |   Z  |   X  |   C  |   D  |   V  ||   K  |   H  |   ,  |   .  |   /  | )    ] |
	`--------+------+------+------+------+------'`------+------+------+------+------+--------'
	         |   `  |      |   [  |   ]  |              | Left | Down |  Up  | Right|
	         `---------------------------'              `---------------------------'
	                             ,--------------.,--------------.
	                             |Ctl/Esc| LAlt || RAlt | RCtl  |
	                      ,------|-------|------||------+-------+-------.
	                      |      | Enter |Number|| RGUI | Delete|       |
	                      | Space|   /   |------||------|   /   | Bspc  |
	                      |  /Fn | LShift| Bspc || Media| RShift|  /Fn  |
	                      `---------------------'`----------------------' 


        QWERTY layer:
	,-------------------------------------------.,-------------------------------------------.
	|   =    |   1  |   2  |   3  |   4  |   5  ||   6  |   7  |   8  |   9  |   0  |   -    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| Tab    |   Q  |   W  |   E  |   R  |   T  ||   Y  |   U  |   I  |   O  |   P  |   \    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| Ctl/Esc|   A  |   S  |   D  |   F  |   G  ||   H  |   J  |   K  |   L  |   ;  |   '    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| (    [ |   Z  |   X  |   C  |   V  |   B  ||   N  |   M  |   ,  |   .  |   /  | )    ] |
	`--------+------+------+------+------+------'`------+------+------+------+------+--------'
	         |   `  |      |   [  |   ]  |              | Left | Down |  Up  | Right|
	         `---------------------------'              `---------------------------'
	                             ,--------------.,--------------.
	                             |Ctl/Esc| LAlt || RAlt | RCtl  |
	                      ,------|-------|------||------+-------+-------.
	                      |      | Enter |Number|| RGUI | Delete|       |
	                      | Space|   /   |------||------|   /   | Bspc  |
	                      |  /Fn | LShift| Bspc || Media| RShift|  /Fn  |
	                      `---------------------'`----------------------' 


        Colemak Mod-DH Gaming layer:
	,-------------------------------------------.,-------------------------------------------.
	|   =    |   1  |   2  |   3  |   4  |   5  ||   6  |   7  |   8  |   9  |   0  |   -    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| Tab    |   Q  |   W  |   F  |   P  |   B  ||   J  |   L  |   U  |   Y  |   ;  |   \    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| Ctl/Esc|   A  |   R  |   S  |   T  |   G  ||   M  |   N  |   E  |   I  |   O  |   '    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| LShift |   Z  |   X  |   C  |   D  |   V  ||   K  |   H  |   ,  |   .  |   /  | RShift |
	`--------+------+------+------+------+------'`------+------+------+------+------+--------'
	         |   `  |      |   [  |   ]  |              | Left | Down |  Up  | Right|
	         `---------------------------'              `---------------------------'
	                             ,--------------.,--------------.
	                             |Ctl/Esc| LAlt || RAlt | RCtl  |
	                      ,------|-------|------||------+-------+-------.
	                      |      | Enter |Number|| RGUI | Delete|       |
	                      | Space|   /   |------||------|   /   | Bspc  |
	                      |      | LShift| Bspc || Media| RShift|       |
	                      `---------------------'`----------------------'


         QWERTY Gaming layer:
	,-------------------------------------------.,-------------------------------------------.
	|   =    |   1  |   2  |   3  |   4  |   5  ||   6  |   7  |   8  |   9  |   0  |   -    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| Tab    |   Q  |   W  |   E  |   R  |   T  ||   Y  |   U  |   I  |   O  |   P  |   \    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| Ctl/Esc|   A  |   S  |   D  |   F  |   G  ||   H  |   J  |   K  |   L  |   ;  |   '    |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	| LShift |   Z  |   X  |   C  |   V  |   B  ||   N  |   M  |   ,  |   .  |   /  | RShift |
	`--------+------+------+------+------+------'`------+------+------+------+------+--------'
	         |   `  |      |   [  |   ]  |              | Left | Down |  Up  | Right|
	         `---------------------------'              `---------------------------'
	                             ,--------------.,--------------.
	                             |Ctl/Esc| LAlt || RAlt | RCtl  |
	                      ,------|-------|------||------+-------+-------.
	                      |      | Enter |Number|| RGUI | Delete|       |
	                      | Space|   /   |------||------|   /   | Bspc  |
	                      |      | LShift| Bspc || Media| RShift|       |
	                      `---------------------'`----------------------'


        Media layer:
	,-------------------------------------------.,-------------------------------------------.
	|        |      |      |      |      |      ||      |      |      |      |      |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      |      |      |      |      ||      |      |      |      |      |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      | Mute | Vol- | Vol+ |      ||      |      |      |      |      |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      | Prev | Play | Next | Stop ||      |      |      |      |      |        |
	`--------+------+------+------+------+------'`------+------+------+------+------+--------'
	         |      |      |      |      |              |      |      |      |      |
	         `---------------------------'              `---------------------------'
	                              ,-------------.,-------------.
	                              |      |      ||      |      |
	                       ,------|------|------||------+------+------.
	                       |      |      |      ||      |      |      |
	                       |      |      |------||------|      |      |
	                       |      |      |      ||      |      |      |
	                       `--------------------'`--------------------'

        Keypad layer:
	,-------------------------------------------.,-------------------------------------------.
	|        |      |      |      |      |      ||      |      | KP = | KP / | KP * |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      |      |  Up  |      |      ||      | KP 7 | KP 8 | KP 9 | KP - |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      | Left | Down | Right|      ||      | KP 4 | KP 5 | KP 6 | KP + |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      |      |      |      |      ||      | KP 1 | KP 2 | KP 3 |KP Ent|        |
	`--------+------+------+------+------+------'`------+------+------+------+------+--------'
	         |      | INS  |      |      |              |      |      | KP . |KP Ent|
	         `---------------------------'              `---------------------------'
	                              ,-------------.,-------------.
	                              |      |      ||      |      |
	                       ,------|------|------||------+------+------.
	                       |      |      |      ||      |      |      |
	                       |      |      |------||------|      | KP 0 |
	                       |      |      |      ||      |      |      |
	                       `--------------------'`--------------------'

        Layer switch layer:
	,-------------------------------------------.,-------------------------------------------.
	|        |Col DH|QWERTY|Col GM|QW GM |      ||      |      |      |      |      |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      |      |      |      |      ||      |      |      |      |      |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      |      |      |      |      ||      |      |      |      |      |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      |      |      |      |      ||      |      |      |      |      |        |
	`--------+------+------+------+------+------'`------+------+------+------+------+--------'
	         |      |      |      |      |              |      |      |      |      |
	         `---------------------------'              `---------------------------'
	                              ,-------------.,-------------.
	                              |      |      ||      |      |
	                       ,------|------|------||------+------+------.
	                       |      |      |      ||      |      |      |
	                       |      |      |------||------|      |      |
	                       |      |      |      ||      |      |      |
	                       `--------------------'`--------------------'

        Function layer:
	,-------------------------------------------.,-------------------------------------------.
	|   F11  |  F1  |  F2  |  F3  |  F4  |  F5  ||  F6  |  F7  |  F8  |  F9  |  F10 |  F12   |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      |      |  Up  |      |      ||      | Home |  Up  |  End |      |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      | Left | Down | Right|      || PgUp | Right| Down | Left |      |        |
	|--------+------+------+------+------+------||------+------+------+------+------+--------|
	|        |      |      |      |      |      || PgDn |      |      |      |      |        |
	`--------+------+------+------+------+------'`------+------+------+------+------+--------'
	         |      |  INS |      |      |              |      |      |      |      |
	         `---------------------------'              `---------------------------'
	                              ,-------------.,-------------.
	                              |      |      ||      |      |
	                       ,------|------|------||------+------+------.
	                       |      |      |      ||      |      |      |
	                       |      |      |------||------|      |      |
	                       |      |      |      ||      |      |      |
	                       `--------------------'`--------------------'


*/

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

[_CMD] = KEYMAP(
           // Left Hand
           KC_ESC,        KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,
           KC_EQL,        KC_1,    KC_2,    KC_3,    KC_4,    KC_5,
           KC_TAB,        KC_Q,    KC_W,    KC_F,    KC_P,    KC_B,
           CTL_T(KC_ESC), KC_A,    KC_R,    KC_S,    KC_T,    KC_G,
           TD(LPN_LBC),   KC_Z,    KC_X,    KC_C,    KC_D,    KC_V,
                          KC_GRV,  XXXXXXX, KC_LBRC, KC_RBRC,

           //Left Thumb
                                   CTL_T(KC_ESC),    KC_LALT,
                                                     MO(_NM),
           LT(_FN, KC_SPC),        LSFT_T(KC_ENT),   KC_BSPC,

           //Right Hand
           KC_F9,         KC_F10,  KC_F11,  KC_F12,  KC_PSCR, KC_SLCK, KC_PAUS, TG(_KP), MO(_LY),
           KC_6,          KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS,
           KC_J,          KC_L,    KC_U,    KC_Y,    KC_SCLN, KC_BSLS,
           KC_M,          KC_N,    KC_E,    KC_I,    KC_O,    KC_QUOT,
           KC_K,          KC_H,    KC_COMM, KC_DOT,  KC_SLSH, TD(RPN_RBC),
           KC_LEFT,       KC_DOWN, KC_UP,   KC_RGHT,

           //Right Thumb
           KC_RALT,       KC_RCTL,
           KC_RGUI,
           MO(_MD),       RSFT_T(KC_DEL),   LT(_FN2, KC_BSPC)
    ),

[_QW] = KEYMAP(
           // Left Hand
           KC_ESC,        KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,
           KC_EQL,        KC_1,    KC_2,    KC_3,    KC_4,    KC_5,
           KC_TAB,        KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,
           CTL_T(KC_ESC), KC_A,    KC_S,    KC_D,    KC_F,    KC_G,
           TD(LPN_LBC),   KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,
                          KC_GRV,  XXXXXXX, KC_LBRC, KC_RBRC,
           
           //Left Thumb
                                   CTL_T(KC_ESC),    KC_LALT,
                                                     MO(_NM),
           LT(_FN, KC_SPC),        LSFT_T(KC_ENT),   KC_BSPC,

           //Right Hand
           KC_F9,         KC_F10,  KC_F11,  KC_F12,  KC_PSCR, KC_SLCK, KC_PAUS, TG(_KP), MO(_LY),
           KC_6,          KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS,
           KC_Y,          KC_U,    KC_I,    KC_O,    KC_P,    KC_BSLS,
           KC_H,          KC_J,    KC_K,    KC_L,    KC_SCLN, KC_QUOT,
           KC_N,          KC_M,    KC_COMM, KC_DOT,  KC_SLSH, TD(RPN_RBC),
           KC_LEFT,       KC_DOWN, KC_UP,   KC_RGHT,

           //Right Thumb
           KC_RALT,       KC_RCTL,
           KC_RGUI,
           MO(_MD),       RSFT_T(KC_DEL),   LT(_FN2, KC_BSPC)

    ),

[_CG] = KEYMAP(
           // Left Hand
           KC_ESC,        KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,
           KC_EQL,        KC_1,    KC_2,    KC_3,    KC_4,    KC_5,
           KC_TAB,        KC_Q,    KC_W,    KC_F,    KC_P,    KC_B,
           CTL_T(KC_ESC), KC_A,    KC_R,    KC_S,    KC_T,    KC_G,
           KC_LSFT,       KC_Z,    KC_X,    KC_C,    KC_D,    KC_V,
                          KC_GRV,  XXXXXXX, KC_LBRC, KC_RBRC,

           //Left Thumb
                                   CTL_T(KC_ESC),    KC_LALT,
                                                     MO(_NM),
           KC_SPC,                 LSFT_T(KC_ENT),   KC_BSPC,

           //Right Hand
           KC_F9,         KC_F10,  KC_F11,  KC_F12,  KC_PSCR, KC_SLCK, KC_PAUS, TG(_KP), MO(_LY),
           KC_6,          KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS,
           KC_J,          KC_L,    KC_U,    KC_Y,    KC_SCLN, KC_BSLS,
           KC_M,          KC_N,    KC_E,    KC_I,    KC_O,    KC_QUOT,
           KC_K,          KC_H,    KC_COMM, KC_DOT,  KC_SLSH, KC_RSFT,
           KC_LEFT,       KC_DOWN, KC_UP,   KC_RGHT,

           //Right Thumb
           KC_RALT,       KC_RCTL,
           KC_RGUI,
           MO(_MD),       RSFT_T(KC_DEL),   KC_BSPC
    ),

[_QG] = KEYMAP(
           // Left Hand
           KC_ESC,        KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,
           KC_EQL,        KC_1,    KC_2,    KC_3,    KC_4,    KC_5,
           KC_TAB,        KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,
           CTL_T(KC_ESC), KC_A,    KC_S,    KC_D,    KC_F,    KC_G,
           KC_LSFT,       KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,
                          KC_GRV,  XXXXXXX, KC_LBRC, KC_RBRC,

           //Left Thumb
                                   CTL_T(KC_ESC),    KC_LALT,
                                                     MO(_NM),
           KC_SPC,                 LSFT_T(KC_ENT),   KC_BSPC,

           //Right Hand
           KC_F9,         KC_F10,  KC_F11,  KC_F12,  KC_PSCR, KC_SLCK, KC_PAUS, TG(_KP), MO(_LY),
           KC_6,          KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS,
           KC_Y,          KC_U,    KC_I,    KC_O,    KC_P,    KC_BSLS,
           KC_H,          KC_J,    KC_K,    KC_L,    KC_SCLN, KC_QUOT,
           KC_N,          KC_M,    KC_COMM, KC_DOT,  KC_SLSH, KC_RSFT,
           KC_LEFT,       KC_DOWN, KC_UP,   KC_RGHT,

           //Right Thumb
           KC_RALT,       KC_RCTL,
           KC_RGUI,
           MO(_MD),       RSFT_T(KC_DEL),   KC_BSPC
    ),

[_NM] = KEYMAP(
           // Left Hand
           _______, _______, _______, _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           KC_EQL,  KC_1,    KC_2,    KC_3,    KC_4,    KC_5,
           _______, _______, _______, _______, _______, _______,
                    _______, _______, _______, _______,

           // Left Thumb
                    _______, _______,
                             _______,
           _______, _______, _______,

           // Right Hand
           _______, _______, _______, _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_MINS,
           _______, _______, _______, _______, _______, _______,
                    _______, _______, _______, _______, 

           // Right Thumb
           _______, _______,
           _______,
           _______, _______, _______
    ),

[_MD] = KEYMAP(
           // Left Hand
           _______, _______, _______, _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           _______, _______, KC_MUTE, KC_VOLD, KC_VOLU, _______,
           _______, _______, KC_MPRV, KC_MPLY, KC_MNXT, KC_MSTP,
                    _______, _______, _______, _______,

           // Left Thumb
                    _______, _______,
                             _______,
           _______, _______, _______,

           // Right Hand
           _______, _______, _______, _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
                    _______, _______, _______, _______,

           // Right Thumb
           _______, _______,
           _______,
           _______, _______, _______
    ),

[_KP] = KEYMAP(
           // Left Hand
           _______, _______, _______, _______, _______, _______, _______, _______, _______,
           _______, _______, _______, _______, _______, _______,
           _______, _______, _______, KC_UP,   _______, _______,
           _______, _______, KC_LEFT, KC_DOWN, KC_RGHT, _______,
           _______, _______, _______, _______, _______, _______,
                    _______, KC_INS,  _______, _______,

           // Left Thumb
                    _______, _______,
                             _______,
           _______, _______, _______,

           // Right Hand
           _______, _______, _______, _______, _______, _______, _______, _______, _______,
           _______, _______, KC_PEQL, KC_PSLS, KC_PAST, _______,
           _______, KC_P7,   KC_P8,   KC_P9,   KC_PMNS, _______,
           _______, KC_P4,   KC_P5,   KC_P6,   KC_PPLS, _______,
           _______, KC_P1,   KC_P2,   KC_P3,   KC_PENT, _______,
                    _______, _______, KC_PDOT, KC_PENT,

           // Right Thumb
           _______, _______,
           _______,
           _______, _______, KC_P0
    ),

[_LY] = KEYMAP(
           // Left Hand
           _______, _______,  _______, _______, _______, _______, _______, _______, _______,
           _______, DF(_CMD), DF(_QW), DF(_CG), DF(_QG), _______,
           _______, _______,  _______, _______, _______, _______,
           _______, _______,  _______, _______, _______, _______,
           _______, _______,  _______, _______, _______, _______,
                    _______,  _______, _______, _______,

           // Left Thumb
                    _______, _______,
                             _______,
           _______, _______, _______,

           // Right Hand
           _______, _______,  _______, _______, _______, _______, _______, _______, _______,
           _______, _______,  _______, _______, _______, _______,
           _______, _______,  _______, _______, _______, _______,
           _______, _______,  _______, _______, _______, _______,
           _______, _______,  _______, _______, _______, _______,
                    _______,  _______, _______, _______, 

           // Right Thumb
           _______, _______,
           _______,
           _______, _______,  _______
    ),

[_FN] = KEYMAP(
           // Left Hand
           _______, _______, _______, _______, _______, _______, _______, _______, _______,
           KC_F11,  KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,
           _______, _______, _______, KC_UP,   _______, _______,
           _______, _______, KC_LEFT, KC_DOWN, KC_RGHT, _______,
           _______, _______, _______, _______, _______, _______,
                    _______, KC_INS,  _______, _______,

           // Left Thumb
                    _______, _______,
                             _______,
           _______, _______, _______,

           // Right Hand
           _______, _______, _______, _______, _______, _______, _______, _______, RESET,
           KC_F6,   KC_F7,   KC_F8,   KC_F9,   KC_F10,  KC_F12,
           _______, KC_HOME, KC_UP,   KC_END,  _______, _______,
           KC_PGUP, KC_LEFT, KC_DOWN, KC_RGHT, _______, _______,
           KC_PGDN, _______, _______, _______, _______, _______,
                    _______, _______, _______, _______, 

           // Right Thumb
           _______, _______,
           _______,
           _______, _______, _______
    ),

[_FN2] = KEYMAP(
           // Left Hand
           _______, _______, _______, _______, _______, _______, _______, _______, _______,
           KC_F11,  KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,
           _______, _______, _______, KC_UP,   _______, _______,
           _______, _______, KC_LEFT, KC_DOWN, KC_RGHT, _______,
           _______, _______, _______, _______, _______, _______,
                    _______, KC_INS,  _______, _______,

           // Left Thumb
                    _______, _______,
                             _______,
           _______, _______, _______,

           // Right Hand
           _______, _______, _______, _______, _______, _______, _______, _______, RESET,
           KC_F6,   KC_F7,   KC_F8,   KC_F9,   KC_F10,  KC_F12,
           _______, KC_HOME, KC_UP,   KC_END,  _______, _______,
           KC_PGUP, KC_LEFT, KC_DOWN, KC_RGHT, _______, _______,
           KC_PGDN, _______, _______, _______, _______, _______,
                    _______, _______, _______, _______, 

           // Right Thumb
           _______, _______,
           _______,
           _______, _______, _______
    )

};

const uint16_t PROGMEM fn_actions[] = {

};

const macro_t *action_get_macro(keyrecord_t *record, uint8_t id, uint8_t opt)
{
  // MACRODOWN only works in this function
      switch(id) {
        case 0:
          if (record->event.pressed) {
            register_code(KC_RSFT);
          } else {
            unregister_code(KC_RSFT);
          }
        break;
      }
    return MACRO_NONE;
};


void matrix_init_user(void) {

}

void matrix_scan_user(void) {

}

bool process_record_user(uint16_t keycode, keyrecord_t *record) {
  return true;
}

void led_set_user(uint8_t usb_led) {

}
